@@ -45,6 +45,10 @@
 #include "gedit-debug.h"
 #include "gedit-utils.h"
 
+#ifndef ENABLE_GVFS_METADATA
+#include "gedit-metadata-manager.h"
+#endif
+
 typedef struct
 {
 	GeditGioDocumentLoader *loader;
@@ -140,12 +144,6 @@ gedit_gio_document_loader_dispose (GObject *object)
 	G_OBJECT_CLASS (gedit_gio_document_loader_parent_class)->dispose (object);
 }
 
-static void
-gedit_gio_document_loader_finalize (GObject *object)
-{
-	G_OBJECT_CLASS (gedit_gio_document_loader_parent_class)->finalize (object);
-}
-
 static void
 gedit_gio_document_loader_class_init (GeditGioDocumentLoaderClass *klass)
 {
@@ -153,7 +151,6 @@ gedit_gio_document_loader_class_init (GeditGioDocumentLoaderClass *klass)
 	GeditDocumentLoaderClass *loader_class = GEDIT_DOCUMENT_LOADER_CLASS (klass);
 
 	object_class->dispose = gedit_gio_document_loader_dispose;
-	object_class->finalize = gedit_gio_document_loader_finalize;
 
 	loader_class->load = gedit_gio_document_loader_load;
 	loader_class->cancel = gedit_gio_document_loader_cancel;
@@ -196,7 +193,7 @@ get_metadata_encoding (GeditDocumentLoader *loader)
 {
 	const GeditEncoding *enc = NULL;
 
-#ifdef G_OS_WIN32
+#ifndef ENABLE_GVFS_METADATA
 	gchar *charset;
 	const gchar *uri;
 
@@ -234,11 +231,14 @@ get_metadata_encoding (GeditDocumentLoader *loader)
 }
 
 static void
-remote_load_completed_or_failed (GeditGioDocumentLoader *gvloader, AsyncData *async)
+remote_load_completed_or_failed (GeditGioDocumentLoader *loader, AsyncData *async)
 {
-	gedit_document_loader_loading (GEDIT_DOCUMENT_LOADER (async->loader),
+	gedit_document_loader_loading (GEDIT_DOCUMENT_LOADER (loader),
 				       TRUE,
-				       async->loader->priv->error);
+				       loader->priv->error);
+
+	if (async)
+		async_data_free (async);
 }
 
 static void
@@ -254,7 +254,9 @@ close_input_stream_ready_cb (GInputStream *stream,
 			     AsyncData     *async)
 {
 	GError *error = NULL;
-	
+
+	gedit_debug (DEBUG_LOADER);
+
 	/* check cancelled state manually */
 	if (g_cancellable_is_cancelled (async->cancellable))
 	{
@@ -343,21 +345,21 @@ async_read_cb (GInputStream *stream,
 {
 	gedit_debug (DEBUG_LOADER);
 	GeditGioDocumentLoader *gvloader;
-	GeditDocumentLoader *loader;
 	GError *error = NULL;
 
-	gvloader = async->loader;
-	loader = GEDIT_DOCUMENT_LOADER (gvloader);
+	gedit_debug (DEBUG_LOADER);
 
 	/* manually check cancelled state */
 	if (g_cancellable_is_cancelled (async->cancellable))
 	{
-		remote_load_completed_or_failed (gvloader, async);
+		async_data_free (async);
 		return;
 	}
 
+	gvloader = async->loader;
+
 	async->read = g_input_stream_read_finish (stream, res, &error);
-	
+
 	/* error occurred */
 	if (async->read == -1)
 	{
@@ -383,7 +385,11 @@ async_read_cb (GInputStream *stream,
 	/* end of the file, we are done! */
 	if (async->read == 0)
 	{
-		GEDIT_DOCUMENT_LOADER (gvloader)->auto_detected_encoding =
+		GeditDocumentLoader *loader;
+
+		loader = GEDIT_DOCUMENT_LOADER (gvloader);
+
+		loader->auto_detected_encoding =
 			gedit_smart_charset_converter_get_guessed (gvloader->priv->converter);
 
 		loader->auto_detected_encoding = gedit_smart_charset_converter_get_guessed (gvloader->priv->converter);
@@ -449,7 +455,6 @@ finish_query_info (AsyncData *async)
 {
 	GeditGioDocumentLoader *gvloader;
 	GeditDocumentLoader *loader;
-	GInputStream *utf8_stream;
 	GInputStream *conv_stream;
 	GFileInfo *info;
 	GSList *candidate_encodings;
@@ -489,10 +494,7 @@ finish_query_info (AsyncData *async)
 						    G_CONVERTER (gvloader->priv->converter));
 	g_object_unref (gvloader->priv->stream);
 
-	utf8_stream = g_utf8_input_stream_new (conv_stream);
-	g_object_unref (conv_stream);
-
-	gvloader->priv->stream = utf8_stream;
+	gvloader->priv->stream = conv_stream;
 
 	/* Output stream */
 	gvloader->priv->output = gedit_document_output_stream_new (loader->document);
@@ -545,7 +547,9 @@ mount_ready_callback (GFile        *file,
 {
 	GError *error = NULL;
 	gboolean mounted;
-	
+
+	gedit_debug (DEBUG_LOADER);
+
 	/* manual check for cancelled state */
 	if (g_cancellable_is_cancelled (async->cancellable))
 	{
